
# q-expansion of newform 525.3.s.g, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 525
weight = 3
poly_data = [1, 0, 0, 0, -1, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 1, 0, 0, 0, 0, 0], 1], [[0, 0, 0, 0, 1, 0, 0, 0], 1], [[0, 0, 0, 0, 0, 0, 1, 0], 1], [[0, 2, 0, 1, 0, -1, 0, 1], 1], [[0, -1, 0, 2, 0, 2, 0, -1], 1], [[0, 1, 0, 1, 0, 1, 0, -2], 1], [[0, 1, 0, -1, 0, 1, 0, 2], 1]]

hecke_ring_character_values = [[176, [1, 0, 0, 0, 0, 0, 0, 0]], [127, [-1, 0, 0, 0, 0, 0, 0, 0]], [451, [1, 0, -1, 0, 0, 0, 0, 0]]]
aps_data = [[0, 1, 0, 0, -1, 0, 0, 0], [0, 1, 0, 1, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 3, 0, 5, 0, 0, 0, 0], [0, 0, 2, 0, 0, 2, 0, 0], [0, -10, 0, 5, 4, 0, 0, -2], [0, -6, 0, -6, -2, 0, 0, -2], [-5, 0, -5, 0, 0, -6, -6, 0], [0, 20, 0, 0, 0, 0, 0, 0], [2, 0, 0, 0, 0, 0, -12, 0], [0, 0, 0, 0, 0, 8, -16, 0], [0, -25, 0, 0, 10, 0, 0, 0], [10, 0, -20, 0, 0, 28, -14, 0], [0, 0, 0, 50, 0, 0, 0, -4], [0, -24, 0, 48, 6, 0, 0, -12], [0, 22, 0, -22, -30, 0, 0, 30], [8, 0, -4, 0, 0, -18, 36, 0], [-39, 0, -39, 0, 0, -6, -6, 0], [0, 1, 0, -1, -38, 0, 0, 38], [-26, 0, 0, 0, 0, 0, -20, 0], [0, -15, 0, -15, 4, 0, 0, 4], [-19, 0, 19, 0, 0, 16, -16, 0], [0, 48, 0, -24, 20, 0, 0, -10], [12, 0, 12, 0, 0, 10, 10, 0], [0, -34, 0, 17, -48, 0, 0, 24], [36, 0, -18, 0, 0, -2, 4, 0], [0, 41, 0, -82, -4, 0, 0, 8], [0, 166, 0, 0, -6, 0, 0, 0], [0, 0, -49, 0, 0, -6, 0, 0], [0, 0, 0, 70, 0, 0, 0, 32], [0, 0, 0, 151, 0, 0, 0, -4], [20, 0, 20, 0, 0, -4, -4, 0], [0, -244, 0, 244, 12, 0, 0, -12], [47, 0, -94, 0, 0, 68, -34, 0], [10, 0, -10, 0, 0, -86, 86, 0], [0, 0, -107, 0, 0, 52, 0, 0], [0, -29, 0, -29, -2, 0, 0, -2], [0, -119, 0, 0, -66, 0, 0, 0], [0, -20, 0, 10, 68, 0, 0, -34], [0, 110, 0, -220, 12, 0, 0, -24], [0, 0, -134, 0, 0, -24, 0, 0], [-44, 0, 88, 0, 0, 56, -28, 0], [146, 0, -146, 0, 0, -54, 54, 0], [0, 26, 0, -26, 36, 0, 0, -36], [0, 0, 0, 62, 0, 0, 0, 32], [-46, 0, 23, 0, 0, -56, 112, 0], [137, 0, 0, 0, 0, 0, 30, 0], [0, 6, 0, -3, 144, 0, 0, -72], [0, 98, 0, 98, 24, 0, 0, 24], [-21, 0, -21, 0, 0, 74, 74, 0], [0, 98, 0, 0, 78, 0, 0, 0], [-64, 0, 0, 0, 0, 0, -72, 0], [-202, 0, 101, 0, 0, -52, 104, 0], [-8, 0, 16, 0, 0, 60, -30, 0], [0, -104, 0, 208, 2, 0, 0, -4], [0, -176, 0, 176, -80, 0, 0, 80], [64, 0, -32, 0, 0, 78, -156, 0], [-140, 0, -140, 0, 0, -48, -48, 0], [0, -101, 0, 101, 174, 0, 0, -174], [82, 0, 0, 0, 0, 0, -98, 0], [0, -21, 0, -21, 102, 0, 0, 102], [0, -52, 0, 26, -112, 0, 0, 56], [0, -96, 0, 48, -56, 0, 0, 28], [20, 0, -10, 0, 0, -52, 104, 0], [0, 44, 0, -88, 32, 0, 0, -64], [0, -218, 0, 0, 84, 0, 0, 0], [475, 0, -475, 0, 0, -30, 30, 0], [0, 0, 0, -122, 0, 0, 0, 132], [0, -82, 0, 82, 98, 0, 0, -98], [-124, 0, 248, 0, 0, 80, -40, 0], [0, 234, 0, 234, -58, 0, 0, -58], [-296, 0, 296, 0, 0, 118, -118, 0], [0, 196, 0, 196, -40, 0, 0, -40], [0, 199, 0, 0, 154, 0, 0, 0], [211, 0, 0, 0, 0, 0, -2, 0], [0, 10, 0, -20, 44, 0, 0, -88], [0, 0, 244, 0, 0, 78, 0, 0], [0, 92, 0, -184, -56, 0, 0, 112], [80, 0, -80, 0, 0, -202, 202, 0], [-274, 0, 137, 0, 0, -64, 128, 0], [124, 0, -248, 0, 0, -212, 106, 0], [-631, 0, 0, 0, 0, 0, -38, 0], [0, 0, 290, 0, 0, 132, 0, 0], [0, 512, 0, -256, 0, 0, 0, 0], [165, 0, 165, 0, 0, -20, -20, 0], [0, -610, 0, 0, 72, 0, 0, 0], [-304, 0, 0, 0, 0, 0, -86, 0], [0, 401, 0, 0, -52, 0, 0, 0], [-216, 0, 432, 0, 0, -168, 84, 0], [0, 0, 0, -223, 0, 0, 0, -180], [0, 2, 0, -4, 48, 0, 0, -96], [-392, 0, 196, 0, 0, 18, -36, 0], [0, -134, 0, 134, 68, 0, 0, -68], [562, 0, 0, 0, 0, 0, 120, 0], [-103, 0, 103, 0, 0, 102, -102, 0], [0, -452, 0, 226, -136, 0, 0, 68], [450, 0, 450, 0, 0, -28, -28, 0], [592, 0, -296, 0, 0, -8, 16, 0], [0, -196, 0, 392, 68, 0, 0, -136], [-377, 0, 377, 0, 0, 118, -118, 0], [0, 0, 0, -146, 0, 0, 0, 368], [0, -838, 0, 838, -56, 0, 0, 56], [0, -444, 0, -444, -70, 0, 0, -70], [-530, 0, 530, 0, 0, 142, -142, 0], [0, 0, -275, 0, 0, 150, 0, 0], [0, -100, 0, -100, 136, 0, 0, 136], [0, 832, 0, -416, -120, 0, 0, 60], [0, 254, 0, -508, 56, 0, 0, -112], [0, 0, -74, 0, 0, 290, 0, 0], [11, 0, -22, 0, 0, -128, 64, 0], [0, -25, 0, 50, -60, 0, 0, 120], [0, 326, 0, -326, -136, 0, 0, 136], [0, 0, 0, 50, 0, 0, 0, 84], [136, 0, -68, 0, 0, -176, 352, 0], [197, 0, 0, 0, 0, 0, 132, 0], [0, 0, 296, 0, 0, 172, 0, 0], [0, -210, 0, 105, -508, 0, 0, 254], [0, 136, 0, 136, -170, 0, 0, -170], [0, 314, 0, 0, -366, 0, 0, 0], [668, 0, 0, 0, 0, 0, -30, 0], [258, 0, -129, 0, 0, 34, -68, 0], [0, 0, 0, -211, 0, 0, 0, -168], [0, 284, 0, -568, -140, 0, 0, 280], [0, 220, 0, -220, 106, 0, 0, -106], [-393, 0, -393, 0, 0, -150, -150, 0], [160, 0, 0, 0, 0, 0, -102, 0], [-589, 0, 589, 0, 0, 54, -54, 0], [-74, 0, -74, 0, 0, -46, -46, 0], [0, 834, 0, -417, 136, 0, 0, -68], [0, -137, 0, 274, 14, 0, 0, -28], [0, 0, 467, 0, 0, -274, 0, 0], [0, 0, 0, 508, 0, 0, 0, 72], [229, 0, -229, 0, 0, 300, -300, 0], [0, 0, 0, 121, 0, 0, 0, -114], [390, 0, 390, 0, 0, 98, 98, 0], [108, 0, -216, 0, 0, -32, 16, 0], [0, 268, 0, 268, 208, 0, 0, 208], [0, -391, 0, -391, -122, 0, 0, -122], [0, 920, 0, -460, 88, 0, 0, -44], [0, 0, 580, 0, 0, 208, 0, 0], [-619, 0, 1238, 0, 0, 196, -98, 0], [302, 0, -302, 0, 0, 258, -258, 0], [0, -331, 0, 331, -60, 0, 0, 60], [0, 0, 0, -712, 0, 0, 0, 44], [-474, 0, 237, 0, 0, 18, -36, 0], [812, 0, -1624, 0, 0, -104, 52, 0], [0, 1440, 0, -720, 72, 0, 0, -36], [0, 282, 0, 282, -80, 0, 0, -80], [132, 0, 132, 0, 0, -40, -40, 0], [0, -418, 0, 0, 62, 0, 0, 0], [0, -787, 0, 0, -126, 0, 0, 0], [-246, 0, 492, 0, 0, 376, -188, 0], [0, 0, 0, 857, 0, 0, 0, 94], [0, -542, 0, 1084, -64, 0, 0, 128], [0, 523, 0, -523, 438, 0, 0, -438], [-644, 0, 0, 0, 0, 0, -444, 0], [-274, 0, 274, 0, 0, -244, 244, 0], [-32, 0, -32, 0, 0, -310, -310, 0], [0, -760, 0, 380, -184, 0, 0, 92], [244, 0, -122, 0, 0, -120, 240, 0], [0, -614, 0, 0, 468, 0, 0, 0], [0, 0, 0, 1012, 0, 0, 0, 258], [0, 0, 0, 127, 0, 0, 0, -192], [576, 0, 576, 0, 0, -94, -94, 0], [0, -904, 0, 904, 354, 0, 0, -354], [0, 194, 0, 194, -186, 0, 0, -186], [0, 0, -542, 0, 0, -424, 0, 0], [0, 37, 0, 37, -102, 0, 0, -102]]
